/*NB_REVISION*/

/*NB_COPYRIGHT*/

/*------------------------------------------------------------------------------
 * MCF441X NON-MASKABLE IRQ 7 EXAMPLE
 * This example program uses a GPIO pin to trigger a level 7 interrupt
 * pin. A jumper must be placed from the GPIO pin to the interrupt
 * input pin. To run the example use MTTTY or some other serial terminal to
 * interact with the menu to trigger the interrupt and view the interrupt
 * counter.
 *
 * The MCF54415 processor is used on both the MOD54415 and NANO54415 modules.
 * The table below list the GPIO jumper configuration.
 *
 * MOD54415:
 *    GPIO signal pin: J2.33
 *    IRQ 7: pin 48
 *
 * NANO54415, and dev board revision 1.3:
 *    GPIO signal:  Nano pin 12 = Dev board pin P2.12
 *    IRQ 7: Nano pin 9 = Dev board pin P1.9
 *
 *
 *----------------------------------------------------------------------------*/
#include "predef.h"
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <smarttrap.h>
#include <pins.h>
#include <pin_irq.h>
#include <sim5441x.h>
#include <NetworkDebug.h>

extern "C" {
    void UserMain(void * pd);
    void NMI_C_Part(); // The part of the ISR written in C/C++
    void NMI_ASM_Part(); // The part of the ISR written in assembly
    void SetIntc(int intCntlNum,  long func, int vector, int irqLevel);
}

#ifdef MOD5441X
    const char * AppName = "MOD5441x IRQ 7";
#elif NANO54415
    const char * AppName = "NANO54415 IRQ 7";
#endif

unsigned int IrqCounter;


/*---------------------------------------------------------------------
 * Function_Holder - This is just a place holder so we can create some
 * inline assembly language code. It will never be called. Note that in
 * the __asm__ statements, you need a leading space between (" and the
 * instruction. However, a space is not used for labels.
 *---------------------------------------------------------------------*/
void Function_Holder()
{
// Export the label for ASM part
__asm__ (" .global NMI_ASM_Part ");
// Label for the C part of the ISR
__asm__ (" .extern NMI_C_Part ");
// Label for the assembly part of the ISR
__asm__ ("NMI_ASM_Part:");
// Set the IRQ mask to mask all
__asm__ (" move.w #0x2700,%sr ");
// Make space on the system stack
__asm__ (" lea -60(%a7),%a7 ");
// Save all registers
__asm__ (" movem.l %d0-%d7/%a0-%a6,(%a7) ");
__asm__ (" jsr NMI_C_Part ");
__asm__ (" movem.l (%a7),%d0-%d7/%a0-%a6 ");
__asm__ (" lea 60(%a7),%a7 ");
__asm__ (" rte ");
}

/*---------------------------------------------------------------------
 * NMI_C_Part - This is the C/C++ part of the ISR that is called from
 * the assembly code.
 *---------------------------------------------------------------------*/
void NMI_C_Part()
{
   sim2.eport.epfr = 0x80;   // Clear the interrupt flag

#ifdef MOD5441X
   J2[33] = 1;    // Set GPIO pin high
#elif NANO54415
   Pins[12] = 1;  // Set GPIO pin high
#endif
   IrqCounter++;
}


/*---------------------------------------------------------------------
 * ConfigureIrq7
 * This function is nearly identical to the library function SetPinIrq(),
 * which can only be called for interrupts 1 - 6, which are maskable.
 * While some values could be hard-coded instead of variables, the variables
 * are used to make the code more readable as an example.
 *--------------------------------------------------------------------*/
void ConfigureIrq7( int polarity, void ( *func )( void ) )
{
    int irqLevel = 7;      // IRQ priority level
    int intCntlNum = 0;    // Interrupt controller number
    int vector = 7;     // Interrupt source number for interrupt controller
    DWORD mask;

#ifdef MOD5441X
   J2[48].function( PINJ2_48_IRQ7 );
#elif NANO54415
   Pins[9].function( PIN_9_IRQ7 );
#endif

    /* Set irq polarity
       EPPAR settings
        00 Pin IRQn level-sensitive
        01 Pin IRQn rising edge triggered
        10 Pin IRQn falling edge triggered
        11 Pin IRQn falling edge and rising edge triggered
    */
    sim2.eport.eppar &= 0x3FFF;         // Clear IRQ7 bits
    if ( polarity > 0 )
       sim2.eport.eppar |= 0x4000;      // Positive edge trigger
    else if ( polarity < 0 )
       sim2.eport.eppar |= 0x8000;      // Negative edge trigger
    else
       sim2.eport.eppar |= 0xC000;      // Both edges

    sim2.eport.epddr &= ~80;            // Configure IRQ7 pin as an input

    // Set the interrupt level for the corresponding interrupt source
    sim2.intc[intCntlNum].icrn[vector] = ( irqLevel & 7 );

    vector_base.table[( intCntlNum * 64 ) + 64 + vector] = (unsigned long)func;

    sim2.eport.epier |= 0x80;           // Enable interrupt

    // Unmask/enable the level 7 IRQ in the interrupt controller
    mask = ( 1 << vector );
    sim2.intc[intCntlNum].imrl &= ~mask;
}



/*---------------------------------------------------------------------
 * UserMain
 *---------------------------------------------------------------------*/
void UserMain(void * pd)
{
    InitializeStack();

    if (EthernetIP == 0)
    {
        iprintf("Trying DHCP...\r\n");
        GetDHCPAddress();
        iprintf("DHCP assigned the IP address of:  ");
        ShowIP(EthernetIP);
        iprintf("\r\n");
    }
    else
    {
        iprintf("Static IP address:  ");
        ShowIP(EthernetIP);
        iprintf("\r\n");
    }


#ifdef _DEBUG
    InitializeNetworkGDB();
//    InitializeNetworkGDB_and_Wait();
    OSTimeDly(TICKS_PER_SECOND * 5);
#endif

    OSChangePrio(MAIN_PRIO);
    EnableAutoUpdate();
    StartHTTP();

#ifndef _DEBUG
    EnableSmartTraps();
#endif

    IrqCounter = 0;

#ifdef MOD5441X
   // GPIO pin used to trigger IRQ
   iprintf("Configuring J2-33 as GPIO\r\n");
   J2[33].function( PINJ2_33_GPIO );
   J2[33] = 1;
   iprintf("Configuring J2-48 as IRQ7\r\n");
#elif NANO54415
   // GPIO pin used to trigger IRQ
   iprintf("Configuring Pin 12 as GPIO\r\n");
   Pins[12].function( PIN_12_GPIO );
   Pins[12] = 1;
   iprintf("Configuring Pin 9 as IRQ7\r\n");
#else
#error "Platform not suppored in example"
#endif

    ConfigureIrq7( -1, &NMI_ASM_Part);

    while (1)
    {
        iprintf("Any key to generate IRQ\r\n");
        getchar();
#ifdef MOD5441X
        J2[33] = 0;
#elif NANO54415
        Pins[12] = 0;
#endif

        iprintf("Irq Count: %d\r\n", IrqCounter);
    }
}
